###-------------Intro to R and Rstudio------------------------------------------
# 
# By Danielle Walkup (with lots of help from Google) 
# EEB OSOS Workshop, August 2019
#
# This script goes with the intro pdf and will take you through
# Part 1 - An Intro to R and RStudio 
# 
###-------------Familiarizing yourself with Rstudio-----------------------------
## The 4 windows in Rstudio (see associated pdf)
## The top menus (see associated pdf)

## Getting started with Good Coding Practices
# 1) commenting - anything preceded by a '#' will not be evaluated by R
#    Get in the habit of commenting your code, explaining WHY, even if what you 
#    are doing seems obvious at the time. Future you will thank you! 
#
# 2) Use a consistent style within your code (naming conventions, line length,
#    spacing around operators, indentation)
#
# 3) Use meaningful (but short) names: bad: mydata, x; better: lizard.df
#
# 4) Things you can do, but probably shouldn't:
#    Use = instead of <-. Use <- for assignment, and = within functions and definitions
#    Assign new data to reserved functions (e.g. data <- 5, mean <- 8)
#
###---------------Working Directory-------------------------
#
## The default directory when you open RStudio is the Documents folder. But to check your current directory you use:
getwd()

## If you want to see the files currently in the working directory:
dir() 

## If you need to change the working directory: 
setwd("Path-To-The-Directory")  
setwd("C:/Users/danielle.walkup/Dropbox/WorkshopInfo/OSOS_teach/2019IntroR/OSOS_IntroR")
# TIP: the easiest way to do this is to open the folder and copy and paste the 
# file path into the quotes. Note that on a pc, you need to use `/` in the path,
# `\` is used as an escape character in R, so if you copy-paste, make sure you 
# change `\` to `/`.

## Let's double check it worked:
getwd()
#
###---------------Working in RStudio--------------------------------------------
#
## The obligatory R can be used as a calculator tutorial:
## You can do basic arithmetic with R
3+5+8
## It will follow order of operations rules
5*9+14
5*(9+14)
## Other operators:
12^2  #can do exponents
sqrt(100)  #and square roots
pi*3  #there are some built in constants
log(5)  #log in R equals the natural log (i.e. ln)
log10(5)  #this will give you log base 10

## Handy Functions:
## "<-" is the assignment operator: (some example data:)
ex.data <- c(4,6,3,6,7,3,4,6,7,2)
# The c() means to concatenate, where all the numbers are joined together in a 
# vector. We use c() quite a lot.

# We can check to make sure it's there
ex.data

# and check out some of its properties:
mean(ex.data)
min(ex.data)
max(ex.data)
median(ex.data)
sd(ex.data)

## Some basic tools that help you understand and check your objects: 
summary(ex.data)
str(ex.data)
dim(ex.data) #NULL here, better for dataframes or other objects
length(ex.data)  #so we'll try length() instead of dim()
head(ex.data)  #by default returns the 1st 6 things in the object
head(ex.data, 2)  #but we can tell it how many we actually want (> or < 6)
tail(ex.data, 3)
#
###------------Installing and Loading Packages-------------
#
## One of the earlier parts of your script should include a list of packages 
## that will be used in the script. I generally set it up as a commented out
## install.packages() command (because I typically have them already loaded), 
## along with a set of library(), that loads each package

# For example, some packages you might like - you'll use this set tomorrow 
# (and a little later on today) for data manipulation, exploration, and plotting
install.packages("tidyverse") 

# To load each package you use library(package-name) - this allows you to use
# the package during that R session. Besides the base packages, you have to load
# the packages you want to use each session
library(tidyverse)

# and ones we will use later on
install.packages(c("openxlsx", "readxl"))  #note the quotes here
library(openxlsx)  #don't have to have quotes here
library(readxl)
#
###-----------Functions, Loops, Apply---------------------
#
## Packages in R are made up of functions - specific chunks of code that take an 
## object and arguments and return a specific output. Most of the code we have 
## run so far has been made up of functions: `install.packages(), dir(), mean()` 
## are all functions. To find out how to use a specific function, you can search 
## help for more information on a function. Typically, help on a function will 
## return a brief Description; Usage of a function showing the arguments and 
## their defaults; more in-depth explanation of Arguments; a description of the
## output or Value; any relevent References; and finally, Examples.
?lm
args(lm)
example(lm)

# You can also write your own functions in R:
test_function <- function(x, y, z) {
  math <- x + y + (x*y) - z
  print(math)
}

# To call the function, you use the function name and feed it some arguments
# Arguments can be called in order:
test_function(3, 5, 11)
# Or by name:
test_function(x = 5, y = 11, z = 3)

## In addition to functions, there are some other typical programming features 
## that you can use. These include for or while loops, If-Else statments, and 
## R's apply functions. 

# For Loops:
for (i in 1:10) {
  print(sqrt(i))
}

# While Loops:
x <- 4
while(x < 500) {
  print(x)
  x <- x^2
}

# If-else 
if (x > 0) {
  print("x is positive")
} else {
  print("x is negative")
}

## The apply() functions are part of the R base package and are designed to 
## manipulate slices of data from matrices, arrays, lists and dataframes in a 
## repetitive way.

## Some brief examples
# apply(X = data, MARGIN = direction the function is applied, FUN = function to 
# apply)
ex_mat <- matrix(c(1:5,13:17), nrow = 2, ncol = 5)
ex_mat
apply(ex_mat, 1, sum) # the 1 tells apply to sum by row
apply(ex_mat, 2, sum) # the 2 tells apply to sum by column

# lapply works similarly, but the input needs to be a list
ex_list <- list(x = 1:5, y = 45:50, z = 15:20)
lapply(ex_list, sum) # note this one doesn't have margin, because it works 
# within each piece of the list

# sapply works like lapply, but returns a different class of object
sapply(ex_list, sum)
class(lapply(ex_list, sum)) # returns a list
class(sapply(ex_list, sum)) # returns a vector of integers
class(sapply(ex_list, sum, simplify = F)) #however, we can force sapply to 
# return a list and not simplify by adding an additional argument

# mapply is the multivariate version of sapply
mapply(sum, 1:5, 1:5) # applies the function sum to the first element of each 
# vector (i.e., 1 + 1), then the second element (2 + 2), and so on
mapply(sum, 1:5, 5:1)

# tapply allows you to evaluate based on a categorical factor
ex_df <- data.frame(Type = rep(c("red", "blue"),5), x = rnorm(10))
ex_df
tapply(ex_df$x, ex_df$Type, mean) # we take the average of x for each type, 
# red or blue
#