###------------Data Import, Clean, Export---------------------------------------
# 
# By Danielle Walkup (with lots of help from Google) 
# EEB OSOS Workshop, August 2019
#
# This script goes with the OSOS-2019 handout, section 3.2
# 
###------------Packages and Directory-------------------------------------------
# 
## Lets be good and set up any packages we are going to use in this script:
#install.packages(c("tidyverse","arsenal","kableExtra"))
library(tidyverse)
library(arsenal)
library(kableExtra)
library(openxlsx)
library(readxl)

## And double check our working directory - Make sure you are in the 
## OSOS_IntroR Folder, not the RScripts folder. 
getwd()

# but we could change it if we needed to
setwd()   
#
###------------Loading Data-----------------------------------------------------
#
## read.csv or read.table are the typical functions we will use to upload data

## read.csv has a ton of options to read in your data set, check out the different arguments:
args(read.csv)
help(read.csv)
# header  - tell it whether or not there is a head row
# col.names - can change the names of the columns
# row.names - or add row names
# colClasses - can specify the column classes
# strip.white - removes leading or trailing white spaces
# blank.lines.skip - skips empty rows
# na.strings - what values should be considered NA

# First, lets make sure the file is where we think it is! And it's an easy way to 
# get our file name
dir("data")  #looks good

# I'll set a file path here (makes it more easily accessible)
#NOTE: by using data/ i'm telling R to look in that folder for the file
liz_csv <- "data/LizardData.csv"  

## If the file is saved as a .csv we can simply read it in:
liz_df <- read.csv(liz_csv)
head(liz_df, 2)  #double check that it worked

# and we can check out our new data frame:
class(liz_df)
head(liz_df)  #look it automatically reads our headings!
str(liz_df)  #tells us the column classes and info about the data.frame
dim(liz_df)
summary(liz_df)  #gives us a brief overview of each column, already, I can see some indicators of a messy data set: check out the Sex, Regen, and the Recap column summaries! 

## If we have a text file, we can still read it in using read.csv
liz_txt <- "data/LizardData.txt"
liz_df_txt <- read.csv(liz_txt, sep = "\t")  #sep = "\t" tells R its a tab-delimited text file
head(liz_df_txt, 2)  #and it looks the same as above!

## Finally, we can also load the excel file using our package openxlsx:
liz_xl <- "data/LizardData.xlsx"
liz_df_xl <- read.xlsx(liz_xl)
head(liz_df_xl, 2)

liz_df_xl2 <- read_excel(liz_xl)
head(liz_df_xl2, 2)

# but note the differences in how each loads the data by default:
str(liz_df)
str(liz_df_txt)
str(liz_df_xl)
str(liz_df_xl2)
#
###------------Loading data from web or repositories----------------------------
#
## You can also import data directly from sources on the internet, if you have 
## the right packages

#install.packages(c("dismo", "rinat", "raster"))
# pull data from gbif (https://www.gbif.org/)
#library(dismo)
#dismo::gbif()

# pull data from iNaturalist (https://www.inaturalist.org/)
#library(rinat)
#rinat::get_inat_obs()

# pull bioclim data (https://www.worldclim.org/bioclim)
#library(raster)
#raster::getData()
#
###------------Load our Practice Data Set---------------------------------------
#
## We've already done this, but let's make sure we have the correct data set loaded and ready to go
## We'll go ahead and add some extra arguments to bring our dataset in cleaner

## I typically like to include 'strip.white = T' in any data that I'm loading in, 
## this removes any leading or trailing white space, because R considers 
## "F", " F", and "F " three different factors.

## Also, since this data set isn't very big, we can just assign colClasses that 
## way each column starts with the correct class, although I will assign date 
## and regen as charactors because of some data entry issues
liz_df <- read.csv(liz_csv, strip.white = T,
                   colClasses = c("factor", "factor", "character", "integer",
                                  "factor", "numeric", "numeric", "character",
                                  "numeric","character", "factor", "character"))

# and check that it loaded ok:
head(liz_df)
tail(liz_df)  #I like to check the end too to make sure there're no blanks
#
###------------Cleaning our Data Set--------------------------------------------
#
# Let's check the summary again, cause we had spotted some issues previously
summary(liz_df) 

# Some known issues:
# Sex - we have some duplicate categories (M vs. Male) and an age class (J)
# Regen - We have No and numbers, that's not good, this should just be numbers
# Recap - again, with the duplicate categories! (N vs no vs No)
# Date is a character here, but we actually want a date. But because people 
# entered the data in multiple formats, it's going to take some work

## Now we can tackle some of the pesky issues:
# let me introduce you to a girl's best friend - the tidyverse! 
# A collection of R packages for data science: https://www.tidyverse.org/
# R for Data Science by Hadley Wickham and Garrett Grolemund: http://r4ds.had.co.nz/
#
# First lets get rid of those no's (and probable yes's) in the Regen
# I'm just going to run a table to see what we have going on
table(liz_df$Regen)  #so we have 82 blanks, 3 No's, 1 Yes, and the weird 38/6
#
# let's check out that weird 38/6, remember the bracket's ([row,col]) we used in 
# the last script to pull out elements of the matrices and data.frames? We can 
# use them here to find the row that matches the one weird record:
liz_df[liz_df$Regen == "38/6", ]  

# so the notes tells me that there were two areas of tail regeneration, so I'm just
# going to change this to 38 using dplyr::recode
liz_df$Regen <- recode(liz_df$Regen, "38/6" = "38")

# we also need to remove the no's and yes's as just blanks
liz_df$Regen <- recode(liz_df$Regen, "No" = "", "Yes" = "") 
# and one last table to double check
table(liz_df$Regen)  
# so lets make that numeric now.
liz_df$Regen <- as.numeric(liz_df$Regen)

## Let's clean up the sex column, because we do want to use that:
table(liz_df$Sex)
# lets change the Male to M - capitilization matters!
liz_df$Sex <- recode(liz_df$Sex, "Male" = "M", "J" = "")  

# TIP: Recode is a nice function because the recode changes the factor levels as 
# well. Sometimes when you are working with factors, the levels can get 
# complicated, so this is convenient

class(liz_df$Sex)  #this is still a factor, and...
levels(liz_df$Sex)  #now we are down to the three levels we wanted

## So now that we have those basic columns cleaned up, lets subset the data frame
## down to just the columns we want to use
head(liz_df, 1)
liz_sub <- liz_df[, c(1:2, 5:9)]  
head(liz_sub, 2)

# Alternative ways to subset:
liz_sub1 <- liz_df[, c("ID", "Mark", "Sex", "SVL", "Tail", "Regen", "Mass")]
liz_sub2 <- subset(liz_df, select = c("ID", "Mark", "Sex", "SVL", "Tail", "Regen", "Mass"))
liz_sub3 <- liz_df %>% select(ID, Mark, Sex, SVL, Tail, Regen, Mass)

## So many NA's - what if we wanted to create a subset of our data frame with 
## no NA's? 
sum(is.na(liz_sub$Sex)) #sum counts FALSE as 0 and TRUE as 1, letting us get 
# a count of na's
liz_subna <- liz_sub[!is.na(liz_sub$Sex), ]
is.na(liz_subna$Sex)  #that's a lot of falses, lets summarize that
sum(is.na(liz_subna$Sex))  
dim(liz_sub)
dim(liz_subna)

# BUT! because this was a factor that had levels assigned, just because it got
# subsetted doesn't mean those factors disappear! One way to get around this is
# to change the column to numeric or character, subset, then change back to factor
levels(liz_subna$Sex)
#
###------------Exporting Cleaned and Summary Data-------------------------------
#
## Finally I'm going to export my clean data frame so that I can use it in other
## Places as needed
dir.create("outputs")  #first I'm going to add a folder to the directory
write.csv(liz_sub, "outputs/LizardData_Clean.csv")

## What if you just want a table of summary information???
liz_summ <- liz_sub %>% group_by(ID) %>%
            summarise(avgSVL = mean(SVL, na.rm = T), sdSVL = sd(SVL, na.rm = T)
                      , maxSVL = max(SVL, na.rm = T), minSVL = min(SVL, na.rm = T)
                      , avgTail = mean(Tail, na.rm=T), sdTail=sd(Tail, na.rm = T) 
                      , avgMass = mean(Mass, na.rm=T), sdMass=sd(Mass, na.rm=T)
                      , Nliz = n())

# and again we can save that summary output:
write.csv(liz_summ, "outputs/LizardSummaryData.csv")

# alternatively, we could print it as a table for word
liz_summ_tbl <- t(liz_summ)
colnames(liz_summ_tbl) <- liz_summ$ID
liz_summ_tbl <- liz_summ_tbl[-1,]

write2word(kable(liz_summ_tbl, "pandoc"), file = "Liz_Summ.doc")
#
###------------Graphing the Data-------------------------------------------------
#
## R has a number of plotting functions, you'll learn more about them tomorrow.

## NOTE: I'm going to attach liz_sub here to make our lives easier:
attach(liz_sub)  # TIP: when you write attach(), go down a couple lines and add 
                 # detach() that way you don't forget to close it out!

# Basic plot types:
?plot  #so we can see the plot format of (x,y,...), where "..." is a myriad of arguments
#
plot(SVL, Tail)  #for a basic scatterplot
plot(Tail ~ SVL)  #we can also use the "~", here the format is y ~ x though
hist(SVL)  #we have different types of plots
boxplot(SVL ~ ID)  #more plots 
plot(SVL ~ ID)  #but R can be smart about formatting the plots based on the data

## Before we go let's export a plot:
?jpeg
dir.create("figures")
jpeg(filename = "figures/SVL_Tail_Scatter.jpg", width = 1200, height = 1200
     , units = "px", res = 300)
plot(Tail ~ SVL, col = ID, xlab = "Snout-vent Length (mm)"
          , ylab = "Tail Length (mm)")
legend(15, 225, legend = levels(ID), fill = 1:6, cex = 0.5)
dev.off()

# we can do other file types too!
png(filename = "figures/SVL_Tail_Scatter.png", width = 1200, height = 1200
     , units = "px", res = 300, bg = "transparent")
plot(Tail ~ SVL, col = ID, xlab = "Snout-vent Length (mm)"
     , ylab = "Tail Length (mm)")
legend(15, 225, legend = levels(ID), fill = 1:6, cex = 0.5)
dev.off()
#
detach(liz_sub)
#